# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings
from django.contrib.auth.models import User


LAYOUT_HOLE_SPLASH = getattr(settings,
                             'WEBGUI_LAYOUT_HOLE_SPLASH',
                             {'desktop' : 2,
                              'tablet' : 2,
                              'phone' : 2})

LAYOUTS_SPLASHSCREEN = getattr(settings,
                               'WEBGUI_LAYOUTS_SPLASHSCREEN',
                               {'desktop' : (3, 3),
                                'tablet' : (3, 3),
                                'phone' :(2, 2)})

LAYOUTS_MENU = getattr(settings,
                       'WEBGUI_LAYOUTS_MENU',
                       {'desktop' : (7, 2),
                        'tablet' : (7, 2),
                        'phone' : (7, 2)}) 

LAYOUTS_SECONDARYMENU = getattr(settings,
                                'WEBGUI_LAYOUTS_SECONDARYMENU',
                                {'desktiop' : (2, 3),
                                 'tablet' : (2, 3),
                                 'phone' : (2, 3)})


class CircularList(list):#TODO utile, spostare da qualche parte comune
    def __getitem__(self, key):
        try:
            return super(CircularList, self).__getitem__(key)
        except IndexError:
            pass
        try:
            index = int(key)
            index = index % self.__len__()
            return super(CircularList, self).__getitem__(index)
        except ValueError:
            raise TypeError
    
class FunctionBox(models.Model):
    """Rappresenta l'elemento della gui legato alla selezione di un gruppo di
    o una specifica funzionalità."""
    nome = models.CharField(max_length=64)
    padre = models.ForeignKey('self', default=None, blank=True, null=True,
                              help_text="Se null rappresenta un gruppo principale \
                              di funzionalità, altrimenti una singola appartentente \
                              al gruppo padre")
    
    image_splash = models.ImageField(upload_to="img/functionbox/splash",
                            help_text="Immagine mostrata nello splash screen.")
    image_menu = models.ImageField(upload_to="img/functionbox/menu",
                            help_text="Immagine mostrata nel menu.")
    image_selected = models.ImageField(upload_to="img/functionbox/selected",
                            help_text="Immagine mostrata nel menu se selezionata.")
    
    ordinal_splash = models.PositiveIntegerField(
                            help_text="Posizione di default nello splash screen.")
    ordinal_menu = models.PositiveIntegerField(
                            help_text="Posizione di default nel menu.")
    
    # se valorizzato indica una url a cui ridirigere invece di seguire il
    # comportamento standard della libreria (un POST a view unica per ogni
    # function box).
    redirect_url_name = models.CharField(max_length=64, blank=True, null=True)
    # se True il response della vista è usato per caricare la view del layout
    # altrimenti sto uscendo dal layout per andare su pagina indipendente.
    render_in_view = models.BooleanField(default=True)

    
    class Meta:
        ordering = ('ordinal_menu', 'nome')
        
    def __unicode__(self):
        return "%s - %s" % (self.ordinal_menu, self.nome)
    
    def get_ordinal_splash(self, user=None):
        """Torna la posizione ordinale del box nello splash screen,
         eventualmente in relazione all'utente che sta visualizzando la gui."""
        if not user:
            return self.ordinal_splash
        raise NotImplementedError
    
    def get_ordinal_menu(self, user=None):
        """Torna la posizione ordinale del box nel menu, eventualmente in
        relazione all'utente che sta visualizzando la gui."""
        if not user:
            return self.ordinal_menu
        raise NotImplementedError

    
class FunctionBoxPosition(models.Model):
    """Rappresenta la posizione di visualizzazione di un elemento in relazione
    all'utente che sta visualizzando la gui"""
    functionbox = models.ForeignKey(FunctionBox)
    user = models.ForeignKey(User)
    
    ordinal_splash = models.PositiveIntegerField(
                            help_text="Posizione nello splash screen.")
    ordinal_menu = models.PositiveIntegerField(
                            help_text="Posizione nel menu.")

class Layout():
    """Restituisce il layout (righe, colonne) da adottare in base 
    al device di visualizzazione."""
    @staticmethod
    def get_griglia_splash(device=None):
        if not device:
            device = 'desktop'
        return LAYOUTS_SPLASHSCREEN[device]

    @staticmethod
    def get_griglia_menu(device=None):
        if not device:
            device = 'desktop'
        return LAYOUTS_MENU[device]
        
    @staticmethod
    def get_griglia_secondary_menu(device=None):
        if not device:
            device = 'desktop'
        return LAYOUTS_SECONDARYMENU[device]
    
    @staticmethod
    def get_functionbox_for_splash(row, col, actual_first_fb=0, shift=0,
                                   device=None, user=None):
        """Restituisce la function box da visualizzare, data una riga e colonna
        del layout, e un eventuale user."""
        if not device:
            device = 'desktop'
        # recupero i pk delle functionbox padri
        qs_fbs = FunctionBox.objects.filter(padre__exact=None
                                              ).order_by('ordinal_splash'
                                              ).values_list('pk', flat=True)

        fbs = []
        for fb in qs_fbs:
            fbs.append(fb)
        fbs = CircularList(fbs)
        counter_modifier = 0 + shift 
        if actual_first_fb > 0:
            counter_modifier = counter_modifier + fbs.index(actual_first_fb)
        if not user:
            try:
                num_row, num_col = Layout.get_griglia_splash(device)
                # conto i riquadri, righe per colonne prima della mia riga
                # più la mia colonna di appartenenza
                # poi mi sposto in base all'attuale primo visualizzato e a
                # quanto voglio scrollare
                riquadro = ( int(row) * num_col ) + int(col)
                counter = ( int(row) * num_col ) + int(col) + counter_modifier
                # se esiste holenum salterò la relativa casella
                holenum = LAYOUT_HOLE_SPLASH[device]
                if holenum and riquadro == holenum:
                    return None
                if holenum and riquadro > holenum:
                    counter = counter - 1
                return FunctionBox.objects.get(pk=fbs[counter])
            except Exception, e:
                print str(e)
        raise NotImplementedError
    
    @staticmethod
    def get_functionbox_for_menu(row, col, actual_first_fb=0, shift=0,
                                 device=None, user=None):
        """Restituisce la function box da visualizzare, data una riga e colonna
        del layout, e un eventuale user."""
        # recupero i pk delle functionbox padri
        qs_fbs = FunctionBox.objects.filter(padre__exact=None
                                              ).order_by('ordinal_menu'
                                              ).values_list('pk', flat=True)

        fbs = []
        for fb in qs_fbs:
            fbs.append(fb)
        fbs = CircularList(fbs)
        counter_modifier = 0 + shift 
        if actual_first_fb > 0:
            counter_modifier = counter_modifier + fbs.index(actual_first_fb)
        if not user:
            try:
                num_row, num_col = Layout.get_griglia_menu(device)
                # conto i riquadri, righe per colonne prima della mia riga
                # più la mia colonna di appartenenza
                # poi mi sposto in base all'attuale primo visualizzato e a
                # quanto voglio scrollare
                riquadro = ( int(row) * num_col ) + int(col)
                counter = ( int(row) * num_col ) + int(col) + counter_modifier
                return FunctionBox.objects.get(pk=fbs[counter])
            except Exception, e:
                print str(e)
        raise NotImplementedError
    
    @staticmethod
    def get_functionbox_for_secondary_menu(row, col, id_padre=None, actual_first_fb=0,
                                           shift=0, device=None, user=None):
        """Restituisce la function box da visualizzare, data una riga e colonna
        del layout, e un eventuale user."""
        # recupero i pk delle functionbox figlie al padre passato
#        if not id_padre:
#            id_padre = 2
        qs_fbs = FunctionBox.objects.filter(padre__pk__exact=id_padre
                                              ).order_by('ordinal_menu'
                                              ).values_list('pk', flat=True)
        fbs = []
        for fb in qs_fbs:
            fbs.append(fb)
        fbs = CircularList(fbs)
        counter_modifier = 0 + shift 
        if actual_first_fb > 0:
            counter_modifier = counter_modifier + fbs.index(actual_first_fb)
        if not user:
            try:
                num_row, num_col = Layout.get_griglia_secondary_menu(device)
                # conto i riquadri, righe per colonne prima della mia riga
                # più la mia colonna di appartenenza
                # poi mi sposto in base all'attuale primo visualizzato e a
                # quanto voglio scrollare
                riquadro = ( int(row) * num_col ) + int(col)
                counter = ( int(row) * num_col ) + int(col) + counter_modifier
                return FunctionBox.objects.get(pk=fbs[counter])
            except Exception, e:
                print str(e)
        raise NotImplementedError
